(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/platform-browser'), require('@angular/core'), require('@angular/forms')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/platform-browser', '@angular/core', '@angular/forms'], factory) :
	(factory((global['handy-angular-components'] = global['handy-angular-components'] || {}),global._angular_platformBrowser,global._angular_core,global._angular_forms));
}(this, (function (exports,_angular_platformBrowser,_angular_core,_angular_forms) { 'use strict';

var HacDropdownFilterPipe = (function () {
    function HacDropdownFilterPipe() {
    }
    /**
     * @param {?} value
     * @param {?} filter
     * @param {?=} apply
     * @return {?}
     */
    HacDropdownFilterPipe.prototype.transform = function (value, filter, apply) {
        if (apply === void 0) { apply = true; }
        if (!filter || !apply)
            return value;
        return value.filter(function (o) { return o.label.indexOf(filter) > -1 || o.key.toString() === filter; });
    };
    return HacDropdownFilterPipe;
}());
HacDropdownFilterPipe.decorators = [
    { type: _angular_core.Injectable },
    { type: _angular_core.Pipe, args: [{
                name: 'hacDropdownFilter'
            },] },
];
/**
 * @nocollapse
 */
HacDropdownFilterPipe.ctorParameters = function () { return []; };

var HacDropdownColumnizerPipe = (function () {
    function HacDropdownColumnizerPipe() {
    }
    /**
     * @param {?} value
     * @param {?=} columns
     * @return {?}
     */
    HacDropdownColumnizerPipe.prototype.transform = function (value, columns) {
        if (columns === void 0) { columns = 1; }
        if (typeof columns !== 'number' || columns === 1)
            return [value];
        var /** @type {?} */ output = [];
        var /** @type {?} */ currentCols = [];
        for (var /** @type {?} */ i = 1; i <= value.length; i++) {
            currentCols.push(value[i - 1]);
            if (i % columns === 0) {
                output.push(currentCols);
                currentCols = [];
            }
        }
        if (currentCols.length > 0) {
            output.push(currentCols);
        }
        return output;
    };
    return HacDropdownColumnizerPipe;
}());
HacDropdownColumnizerPipe.decorators = [
    { type: _angular_core.Injectable },
    { type: _angular_core.Pipe, args: [{
                name: 'hacDropdownColumnizer'
            },] },
];
/**
 * @nocollapse
 */
HacDropdownColumnizerPipe.ctorParameters = function () { return []; };

var HacDropdown = (function () {
    /**
     * @param {?} elementRef
     * @param {?} dropdownFilter
     */
    function HacDropdown(elementRef, dropdownFilter) {
        this.elementRef = elementRef;
        this.dropdownFilter = dropdownFilter;
        this.optionGroups = [];
        this.placeholder = 'Select';
        this.allowEmpty = false;
        this.filtrable = false;
        this.columns = 1;
        this.selectedChange = new _angular_core.EventEmitter();
        this.collapsed = true;
        this.windowHeight = 0;
        this.syncWindowHeight();
    }
    Object.defineProperty(HacDropdown.prototype, "selected", {
        /**
         * @return {?}
         */
        get: function () {
            return this._selected;
        },
        /**
         * @param {?} v
         * @return {?}
         */
        set: function (v) {
            this._selected = v;
            var /** @type {?} */ selection = this.getSelected();
            this.filter = selection ? selection.label : null;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} event
     * @return {?}
     */
    HacDropdown.prototype.onClick = function (event) {
        if (!this.elementRef.nativeElement.contains(event.target)) {
            this.closeDropdown();
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    HacDropdown.prototype.onResize = function (event) {
        this.syncWindowHeight();
    };
    /**
     * @return {?}
     */
    HacDropdown.prototype.getSelected = function () {
        var _this = this;
        if (this.hasGroups()) {
            return this.optionGroups.map(function (f) { return f.options.find(function (o) { return o.key === _this.selected; }); }).find(function (o) { return o != null; });
        }
        return null;
    };
    /**
     * @param {?} key
     * @return {?}
     */
    HacDropdown.prototype.select = function (key) {
        this.selected = key;
        this.closeDropdown();
        this.selectedChange.emit(this.selected);
    };
    /**
     * @return {?}
     */
    HacDropdown.prototype.hasGroups = function () {
        return this.optionGroups && this.optionGroups.length > 0;
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    HacDropdown.prototype.openDropdown = function (e) {
        this.collapsed = false;
        if (e) {
            e.target.focus();
        }
    };
    /**
     * @param {?=} e
     * @return {?}
     */
    HacDropdown.prototype.closeDropdown = function (e) {
        this.collapsed = true;
    };
    /**
     * @return {?}
     */
    HacDropdown.prototype.shouldApplyFilter = function () {
        var /** @type {?} */ selected = this.getSelected();
        return selected && this.filter !== selected.label;
    };
    /**
     * @param {?} e
     * @return {?}
     */
    HacDropdown.prototype.handleKey = function (e) {
        switch (e.keyCode) {
            case 13:
                this.selectFirstMatchingOption();
                break;
            case 27:
                this.blurFilter();
                this.closeDropdown();
                break;
        }
    };
    /**
     * @return {?}
     */
    HacDropdown.prototype.calcDropdownWidth = function () {
        var /** @type {?} */ labelElem = this.elementRef.nativeElement.querySelector('.hac-dd-label');
        var /** @type {?} */ computedStyle = getComputedStyle(labelElem, null);
        var /** @type {?} */ borderLeft = parseFloat(computedStyle.borderLeftWidth.replace('px', ''));
        var /** @type {?} */ borderRight = parseFloat(computedStyle.borderRightWidth.replace('px', ''));
        return labelElem.offsetWidth - borderLeft - borderRight + "px";
    };
    /**
     * @return {?}
     */
    HacDropdown.prototype.calcDropdownHeight = function () {
        var /** @type {?} */ el = this.elementRef.nativeElement.querySelector('.hac-dd-list');
        var /** @type {?} */ pos = this.getPos(el);
        return window.innerHeight - 10 - pos.y + "px";
    };
    /**
     * @return {?}
     */
    HacDropdown.prototype.syncWindowHeight = function () {
        this.windowHeight = window.innerHeight;
    };
    /**
     * @param {?} el
     * @return {?}
     */
    HacDropdown.prototype.getPos = function (el) {
        var /** @type {?} */ lx, /** @type {?} */ ly;
        for (lx = 0, ly = 0; el != null; lx += el.offsetLeft, ly += el.offsetTop, el = el.offsetParent)
            ;
        return { x: lx, y: ly };
    };
    /**
     * @return {?}
     */
    HacDropdown.prototype.blurFilter = function () {
        this.elementRef.nativeElement.querySelector('.hac-dd-filter').blur();
    };
    /**
     * @return {?}
     */
    HacDropdown.prototype.selectFirstMatchingOption = function () {
        for (var _i = 0, _a = this.optionGroups; _i < _a.length; _i++) {
            var group = _a[_i];
            var /** @type {?} */ candidates = this.dropdownFilter.transform(group.options, this.filter);
            if (candidates.length > 0) {
                this.select(candidates[0].key);
                this.blurFilter();
                break;
            }
        }
    };
    return HacDropdown;
}());
/* Dropdown styling dimensions */
HacDropdown.decorators = [
    { type: _angular_core.Component, args: [{
                selector: 'hac-dropdown',
                template: "<div class=\"hac-dd-label\"  (click)=\"openDropdown($event)\"> <input  type=\"text\" class=\"hac-dd-filter\" [placeholder]=\"placeholder\" [(ngModel)]=\"filter\" [readonly]=\"!filtrable\" (keyup)=\"handleKey($event)\" (focus)=\"openDropdown($event)\" (focusOut)=\"closeDropdown($event)\" /> <i class=\"hac-dd-caret hac-dd-caret--down\" aria-hidden=\"true\"></i> </div> <div class=\"hac-dd-list\"  [ngClass]=\"{ 'hac-dd-list--is-collapsed': collapsed }\"  *ngIf=\"hasGroups()\" [style.minWidth]=\"calcDropdownWidth()\" [style.maxHeight]=\"calcDropdownHeight()\"> <div class=\"hac-dd-option\" *ngIf=\"allowEmpty\" (click)=\"select(null)\">{{placeholder}}</div> <div class=\"hac-dd-group\" *ngFor=\"let group of optionGroups\"> <div class=\"hac-dd-grouplabel\" *ngIf=\"group.label\">{{group.label}}</div> <div class=\"hac-dd-optgroup\"> <div class=\"hac-dd-optrow\" *ngFor=\"let optionRow of group.options | hacDropdownFilter: filter : shouldApplyFilter() | hacDropdownColumnizer: columns\" [ngClass]=\"{'hac-dd-optrow--single-col': columns === 1}\"> <div *ngFor=\"let option of optionRow\"  (click)=\"select(option.key)\" class=\"hac-dd-option hac-dd-choice\"  [ngClass]=\"{  'hac-dd-option--is-selected': option.key === selected }\"> {{option.label}} </div> </div> </div> </div> </div>",
                providers: [
                    HacDropdownFilterPipe,
                    HacDropdownColumnizerPipe
                ]
            },] },
];
/**
 * @nocollapse
 */
HacDropdown.ctorParameters = function () { return [
    { type: _angular_core.ElementRef, },
    { type: HacDropdownFilterPipe, },
]; };
HacDropdown.propDecorators = {
    'optionGroups': [{ type: _angular_core.Input },],
    'placeholder': [{ type: _angular_core.Input },],
    'allowEmpty': [{ type: _angular_core.Input },],
    'filtrable': [{ type: _angular_core.Input },],
    'columns': [{ type: _angular_core.Input },],
    'selectedChange': [{ type: _angular_core.Output },],
    'selected': [{ type: _angular_core.Input },],
    'onClick': [{ type: _angular_core.HostListener, args: ['document:click', ['$event'],] },],
    'onResize': [{ type: _angular_core.HostListener, args: ['window:resize', ['$event'],] },],
};

var HacModule = (function () {
    function HacModule() {
    }
    /**
     * @return {?}
     */
    HacModule.forRoot = function () {
        return {
            ngModule: HacModule
        };
    };
    return HacModule;
}());
HacModule.decorators = [
    { type: _angular_core.NgModule, args: [{
                declarations: [
                    HacDropdown,
                    HacDropdownFilterPipe,
                    HacDropdownColumnizerPipe
                ],
                imports: [
                    _angular_platformBrowser.BrowserModule,
                    _angular_forms.FormsModule
                ],
                exports: [
                    HacDropdown
                ]
            },] },
];
/**
 * @nocollapse
 */
HacModule.ctorParameters = function () { return []; };

exports.HacModule = HacModule;
exports.HacDropdown = HacDropdown;

Object.defineProperty(exports, '__esModule', { value: true });

})));
